﻿/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.ComponentModel;
using System.Drawing;

namespace PaintableSimulator.Engine
{
	/// <summary>
	/// Provides settings for the wimulator engine
	/// </summary>
	[Serializable()]
	public class EngineSettings 
	{
		/// <summary>
		/// 
		/// </summary>
		private System.Threading.ThreadPriority simulatorThreadPriority = System.Threading.ThreadPriority.BelowNormal;

		[Browsable(true)]
		[Description("The thread priority of the simulator. Do not set above Normal")]
		[DisplayName("Simulator priority")]
		[Category("Misc")]
		public System.Threading.ThreadPriority SimulatorThreadPriority
		{
			get { return simulatorThreadPriority; }
			set 
			{
				if (value > System.Threading.ThreadPriority.Normal)
					simulatorThreadPriority = System.Threading.ThreadPriority.Normal;
				else
					simulatorThreadPriority = value; 
			}
		}

		/// <summary>
		/// 
		/// </summary>
		private bool loadEqualization;


		/// <summary>
		/// Gets or sets a value indicating whether to equalize simulator load.
		/// </summary>
		/// <value><c>true</c> if load is to be equalized; otherwise, <c>false</c>.</value>
		[Browsable(true)]
		[Description("Gets or sets a value indicating whether to equalize simulator load.")]
		[DisplayName("Load Equalization")]
		[Category("Misc")]
		public bool LoadEqualization
		{
			get { return loadEqualization; }
			set { loadEqualization = value; }
		}


		/// <summary>
		/// 
		/// </summary>
		private uint maximumComputerCount;

		/// <summary>
		/// Gets or sets the maximum computers to be created.
		/// </summary>
		/// <value>The maximum process fragment memory.</value>
		[Browsable(true)]
		[Description("Gets or sets the maximum computers to be created.")]
		[DisplayName("Maximum computer count")]
		[Category("Misc")]
		public uint MaximumComputerCount
		{
			get { return maximumComputerCount; }
			set { maximumComputerCount = value; }
		}


		/// <summary>
		/// 
		/// </summary>
		private uint maximumNeighbourCount;

		/// <summary>
		/// Gets or sets the maximum neighbour count.
		/// </summary>
		/// <value>The maximum neighbour count.</value>
		[Browsable(true)]
		[Description("Gets or sets the maximum neighbour count.")]
		[DisplayName("Maximum Neighbour Count")]
		[Category("Misc")]
		public uint MaximumNeighbourCount
		{
			get { return maximumNeighbourCount; }
			set { maximumNeighbourCount = value; }
		}


		/// <summary>
		/// 
		/// </summary>
		private int simulatorCycleDeviation;

		/// <summary>
		/// Gets or sets the simulator cycle deviation.
		/// </summary>
		/// <value>The simulator cycle deviation.</value>
		[Browsable(true)]
		[Description("Gets or sets the simulator cycle deviation.")]
		[DisplayName("Simulator cycle deviation")]
		[Category("Simulation")]
		public int SimulatorCycleDeviation
		{
			get { return simulatorCycleDeviation; }
			set { simulatorCycleDeviation = value; }
		}


		/// <summary>
		/// 
		/// </summary>
		private uint paintableComputerFailureTime;

		/// <summary>
		/// Gets or sets the paintable computer failure time.
		/// </summary>
		/// <value>The paintable computer failure time.</value>
		[Browsable(true)]
		[Description("Gets or sets the paintable computer failure time.")]
		[DisplayName("Mean time before failure")]
		[Category("Simulation")]
		public uint PaintableComputerFailureTime
		{
			get { return paintableComputerFailureTime; }
			set { paintableComputerFailureTime = value; }
		}

		/// <summary>
		/// 
		/// </summary>
		private uint radioLinkDelay;

		/// <summary>
		/// Gets or sets the radio link delay.
		/// </summary>
		/// <value>The radio link delay.</value>
		[Browsable(true)]
		[Description("Gets or sets the radio link delay.")]
		[DisplayName("Radio link delay")]
		[Category("Transmission")]
		public uint RadioLinkDelay
		{
			get { return radioLinkDelay; }
			set { radioLinkDelay = value; }
		}


		/// <summary>
		/// 
		/// </summary>
		private double radioLinkFailurePercent;

		/// <summary>
		/// Gets or sets the radio link failure percent.
		/// </summary>
		/// <value>The radio link failure percent.</value>
		[Browsable(true)]
		[Description("Gets or sets the radio link failure percent.")]
		[DisplayName("Transmission failure percentage")]
		[Category("Transmission")]
		public double RadioLinkFailurePercent
		{
			get { return radioLinkFailurePercent; }
			set { radioLinkFailurePercent = value; }
		}


		/// <summary>
		/// 
		/// </summary>
		private uint simulatorCycle;

		/// <summary>
		/// Gets or sets the simulator cycle.
		/// </summary>
		/// <value>The simulator cycle.</value>
		[Browsable(true)]
		[Description("Gets or sets the simulator cycle.")]
		[DisplayName("Simulator cycle")]
		[Category("Simulation")]
		public uint SimulatorCycle
		{
			get { return simulatorCycle; }
			set { simulatorCycle = value; }
		}

		/// <summary>
		/// 
		/// </summary>
		private uint spaceBetweenPaintableComputers;

		/// <summary>
		/// Gets or sets the space between paintable computers.
		/// </summary>
		/// <value>The space between paintable computers.</value>
		[Browsable(true)]
		[Description("Gets or sets the space between paintable computers.")]
		[DisplayName("Computer distance")]
		[Category("Misc")]
		public uint SpaceBetweenPaintableComputers
		{
			get { return spaceBetweenPaintableComputers; }
			set { spaceBetweenPaintableComputers = value; }
		}

		/// <summary>
		/// 
		/// </summary>
		private uint wirelessRadius;

		/// <summary>
		/// Gets or sets the wireless radius.
		/// </summary>
		/// <value>The wireless radius.</value>
		[Browsable(true)]
		[Description("Gets or sets the wireless radius.")]
		[DisplayName("Wireless radius")]
		[Category("Transmission")]
		public uint WirelessRadius
		{
			get { return wirelessRadius; }
			set { wirelessRadius = value; }
		}


		private Size wallSizeRatio;

		/// <summary>
		/// Gets or sets the wall size ratio.
		/// </summary>
		/// <value>The wall size ratio.</value>
		public Size WallSizeRatio
		{
			get { return wallSizeRatio; }
			set 
			{
				if ((value.Width >0) && (value.Height >0))
					wallSizeRatio = value; 
			}
		}



		/// <summary>
		/// Initializes the <see cref="EngineSettings"/> class.
		/// </summary>
		public EngineSettings()
		{

			// load initial values into  properties
			simulatorThreadPriority = System.Threading.ThreadPriority.BelowNormal;
			maximumComputerCount = 2000;
			maximumNeighbourCount = 20;
			simulatorCycleDeviation = 10;
			paintableComputerFailureTime = 1000;
			radioLinkDelay = 50;
			radioLinkFailurePercent = 2.0;
			simulatorCycle = 500;
			spaceBetweenPaintableComputers = 20;
			wirelessRadius = 50;
			wallSizeRatio = new Size(5, 3);
		}

	}
}