﻿/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.ComponentModel;
using System.Diagnostics;

namespace PaintableSimulator.Engine.PaintableComputers.OperatingSystem
{
	/// <summary>
	/// Abstract class representing the skeleton of a generic Process Fragment.
	/// 
	/// Every Class must have the Serializable Attribute supplied. The Following are optional:
	/// ProcessFragmentNameAttribute .. user friendly name
	/// ProcessFragmentDescriptionAttribute .. usage description
	/// ProcessFragmentColorizerAttribute .. specifies a colorizer that will be used for visualisation purpouses
	/// </summary>
	[DebuggerStepThrough()]
	[Serializable()]
	public abstract class ProcessFragmentBase
	{
		/// <summary>
		/// 
		/// </summary>
		[NonSerialized()] private IApiInterface api;

		/// <summary>
		/// Gets or sets the Api.
		/// </summary>
		/// <value>The Api.</value>
		[Browsable(false)]
		public IApiInterface Api
		{
			get { return api; }
			set { api = value; }
		}

		/// <summary>
		/// Returns the ID specified by the PFragIdAttribute of the PFragment
		/// </summary>
		[Browsable(false)]
		public abstract uint ProcessFragmentId { get; }


		/// <summary>
		/// This method is called when the PFrag first enters a Paintable.
		/// </summary>
		/// <remarks>
		/// </remarks>
		public abstract void Install();

		/// <summary>
		/// This method is called after the PFrag queued himself for uninstall and the OS approves an uninstall.
		/// </summary>
		/// <param name="remainingTimeslots"></param>
		/// <remarks>
		/// Here you should do some pre uninstall processing.
		/// Removing posts from the homepage is unnecessary because the OS does that for you automatically.
		/// </remarks>
		public abstract void Uninstall(byte remainingTimeslots);

		/// <summary>
		/// This method is called if the PFrag requested a transfer to the neighboring particles and
		/// his request was approved.
		/// </summary>
		/// <param name="copy">The copy of the ProcessFragmentBase.</param>
		public abstract void TransferGranted(ref ProcessFragmentBase copy);

		/// <summary>
		/// This method is called if the PFragment requested a transfer to the neighboring particles and
		/// his request was denied.
		/// </summary>
		public abstract void TransferRefused();

		/// <summary>
		/// The main processing method for Process Fragments
		/// </summary>
		/// <param name="remainingTimeslots">A value indicating the available processing time for the Update method.</param>
		/// <remarks>
		/// The time slot of 0 indicates less time available for processing and 255 more time.
		/// This is just a suggested value, it has no bearing on the execution of the Update method (yet).
		/// </remarks>
		public abstract void Update(byte remainingTimeslots);


		/// <summary>
		/// Initializes a new instance of the <see cref="ProcessFragment"/> class.
		/// </summary>
		protected ProcessFragmentBase()
		{
		}

	}
}