/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Security;
using System.Security.Permissions;
using PaintableSimulator.Engine.PaintableComputers.OperatingSystem;
using System.Xml;

namespace PaintableSimulator.Engine.PaintableComputers
{
	/// <summary>
	/// 
	/// </summary>
	public abstract class PaintableComputerBase : MarshalByRefObject, IApiInterface
	{
		// Create a logger for use in this class
		/// <summary>
		/// 
		/// </summary>
		protected static readonly log4net.ILog log = log4net.LogManager.GetLogger("Computer");
		// NOTE that using System.Reflection.MethodBase.GetCurrentMethod().DeclaringType
		// is equivalent to typeof(LoggingExample) but is more portable
		// i.e. you can copy the code directly into another class without
		// needing to edit the code.

		/// <summary>
		/// 
		/// </summary>
		public const uint DEFAULT_MAXIMUMNEIGHBOURCOUNT = 16;

		#region Private Fields

		/// <summary>
		/// 
		/// </summary>
		protected uint computerId;

		/// <summary>
		/// 
		/// </summary>
		protected Homepage localHomepage;

		/// <summary>
		/// 
		/// </summary>
        protected List<ExtendedHomepageData> mirroredHomepages;


		/// <summary>
		/// Gets the mirrored homepages.
		/// </summary>
		/// <value>The mirrored homepages.</value>
        public List<Homepage> MirroredHomepages
        {
            get 
			{ 
				// anonymous homepage converter
				return mirroredHomepages.ConvertAll<Homepage>(delegate (ExtendedHomepageData ehp)
						{
							return ehp.Homepage;
						}
						);
			}
        }



		/// <summary>
		/// 
		/// </summary>
		protected Queue<NetworkPacket> receivePadQueue;

		/// <summary>
		/// 
		/// </summary>
		protected PriorityQueue<byte, NetworkPacket> sendPadQueue;
		
		/// <summary>
		/// 
		/// </summary>
		protected PriorityQueue<byte, ExtendedProcessFragmentData> uninstallQueue;

		/// <summary>
		/// 
		/// </summary>
		protected uint maximumNeighbourCount;

		/// <summary>
		/// 
		/// </summary>
		private static PermissionSet processFragmentPermissionSet = new PermissionSet(PermissionState.None);

		#endregion

		#region Constructors

		/// <summary>
		/// Initializes the <see cref="T:PaintableComputerBase"/> type.
		/// </summary>
		static PaintableComputerBase()
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="T:AbstractPaintableComputer"/> class.
		/// </summary>
		/// <param name="maximumNeighbourCount">The maximum neighbour count.</param>
		protected PaintableComputerBase(uint maximumNeighbourCount)
		{
			this.maximumNeighbourCount = maximumNeighbourCount;

			// Generates an internal random number, for debug purposes (and pfrag 
			this.computerId = RandomNumberGenerator.GetRandomNumber;

			// home page initialization
			localHomepage = new Homepage();
			mirroredHomepages = new List<ExtendedHomepageData>((int) maximumNeighbourCount);

			// transmission & process queues
			receivePadQueue = new Queue<NetworkPacket>();
			sendPadQueue = new PriorityQueue<byte, NetworkPacket>(PriorityTarget.MaximumFirts);
			uninstallQueue = new PriorityQueue<byte, ExtendedProcessFragmentData>(PriorityTarget.MaximumFirts);

			// enqueue the HP so the others will know we are here
			HomepagePacket hpPacket = new HomepagePacket(localHomepage.Clone(), computerId, 0);
			sendPadQueue.Push(byte.MaxValue, hpPacket);
		}

		#endregion

		/// <summary>
		/// Gets the computer id.
		/// </summary>
		/// <value>The computer id.</value>
		public uint ComputerId
		{
			get { return this.computerId; }
		}


		/// <summary>
		/// Gets the local home page.
		/// </summary>
		/// <value>The local home page.</value>
		public Homepage LocalHomepage
		{
			get { return localHomepage; }
		}


		/// <summary>
		/// Gets the first NetworkPacket from the send queue, or puts 
		/// a new NetworkPacket on the receive queue.
		/// </summary>
		/// <value>The NetworkPacket to be sent/received</value>
		public NetworkPacket TransmissionAntenna
		{
			get
			{
				// return the top packet or null if none
				if (sendPadQueue.Count != 0)
					return sendPadQueue.Pop();
				else
					return null;
			}


			set
			{
				// TODO: implement a fixed size receivePadQueue or even a cyclic queue
				receivePadQueue.Enqueue(value);
			}
		}


		/// <summary>
		/// Cycles this instance.
		/// </summary>
		public abstract void Cycle();


		#region IApiInterface Members


		/// <summary>
		/// Post a string to the HomePage of the specified process fragment at the end.
		/// </summary>
		/// <param name="processFragmentId">ID of the pfrag to post to.</param>
		/// <param name="post">A string containing data.</param>
		/// <returns>
		/// Index of the post or -1 if posting failed.
		/// </returns>
		public abstract int PostEntry(uint processFragmentId, object post);

		/// <summary>
		/// Post a string to the HomePage of the current process fragment at the end.
		/// </summary>
		/// <param name="post">A string containing data.</param>
		/// <returns>
		/// Index of the post or -1 if posting failed.
		/// </returns>
		public abstract int PostEntry(object post);

		/// <summary>
		/// Posts a string to the HomePage of the specified process fragment at the specified index.
		/// </summary>
		/// <param name="processFragmentId">ID of the posting pfrag.</param>
		/// <param name="index">Index of the post to replace</param>
		/// <param name="post">A string containing data.</param>
		/// <returns>
		/// Index of the post or -1 if posting failed.
		/// </returns>
		public abstract int PostEntry(uint processFragmentId, int index, object post);

		/// <summary>
		/// Posts a string to the HomePage of the current process fragment at the specified index.
		/// </summary>
		/// <param name="index">Index of the post to replace</param>
		/// <param name="post">A string containing data.</param>
		/// <returns>
		/// Index of the post or -1 if posting failed.
		/// </returns>
		public abstract int PostEntry(int index, object post);

		/// <summary>
		/// Removes a post from the HomePage of the specified process fragment.
		/// </summary>
		/// <param name="processFragmentId">Id of the posting pfrag.</param>
		/// <param name="index">Index of the post</param>
		/// <returns>True if post removal succeded.</returns>
		public abstract bool RemoveEntry(uint processFragmentId, int index);

		/// <summary>
		/// Removes a post from the HomePage of the current process fragment.
		/// </summary>
		/// <param name="index">Index of the post</param>
		/// <returns>True if the post removal succeded.</returns>
		public abstract bool RemoveEntry(int index);

		/// <summary>
		/// Removes all posts of the specified process fragment.
		/// </summary>
		/// <param name="processFragmentId">ID of the Process Fragment</param>
		/// <returns>The number of posts removed.</returns>
		public abstract int RemoveAllEntries(uint processFragmentId);

		/// <summary>
		/// Removes all posts of the current process fragment.
		/// </summary>
		/// <returns>The number of posts removed.</returns>
		public abstract int RemoveAllEntries();

		/// <summary>
		/// Returns the number of posts of the specified process fragment.
		/// </summary>
		/// <param name="processFragmentId">ID of the Process Fragment</param>
		/// <returns></returns>
		public abstract int GetPostCount(uint processFragmentId);

		/// <summary>
		/// Returns the number of posts of the current process fragment.
		/// </summary>
		/// <returns></returns>
		public abstract int GetPostCount();

		/// <summary>
		/// Returns the post value of the specified process fragment.
		/// </summary>
		/// <param name="processFragmentId">ID of the Process Fragment</param>
		/// <param name="index">Index of the post</param>
		/// <returns>
		/// A Object containing data if it exists; null otherwise
		/// </returns>
		public abstract object GetPost(uint processFragmentId, int index);

		/// <summary>
		/// Returns the post value of the current process fragment.
		/// </summary>
		/// <param name="index">Index of the post</param>
		/// <returns>
		/// A Object containing data if it exists; null otherwise
		/// </returns>
		public abstract object GetPost(int index);

		/// <summary>
		/// This method returns the number of neighboring homepages mirrored in the IO space.
		/// </summary>
		/// <returns>The number of mirrored homepages.</returns>
		public abstract int GetIOSpaceCount();

		/// <summary>
		/// Returns the number of posts in the chosen process fragment mirror space.
		/// </summary>
		/// <param name="mirrorIndex">Index of the mirror</param>
		/// <param name="processFragmentId">ID of the Process Fragment</param>
		/// <returns>
		/// A count of posts or 0 if the specified process fragment has no posts, -1 if the mirror doesn't exist
		/// </returns>
		public abstract int GetIOPostCount(int mirrorIndex, uint processFragmentId);

		/// <summary>
		/// Returns the number of posts in the mirror space for the current process fragment type.
		/// </summary>
		/// <param name="mirrorIndex">Index of the mirror</param>
		/// <returns>
		/// A count of posts or 0 if the specified process fragment has no posts, -1 if the mirror doesn't exist
		/// </returns>
		public abstract int GetIOPostCount(int mirrorIndex);

		/// <summary>
		/// Returns a mirrored post value of the specified process fragment.
		/// </summary>
		/// <param name="mirrorIndex">Index of the mirrored HomePage.</param>
		/// <param name="processFragmentId">Guid of the PFrag</param>
		/// <param name="postIndex">Index of the post.</param>
		/// <returns>The post if it exists or null otherwise</returns>
		public abstract object GetIOPost(int mirrorIndex, uint processFragmentId, int postIndex);

		/// <summary>
		/// Returns a mirrored post value of the current process fragment type.
		/// </summary>
		/// <param name="mirrorIndex">Index of the mirror.</param>
		/// <param name="postIndex">Index of the post.</param>
		/// <returns></returns>
		public abstract object GetIOPost(int mirrorIndex, int postIndex);

		/// <summary>
		/// Queue a process fragment transfer ticket.
		/// </summary>
		/// <param name="mirrorIndex">The index of the mirrored HomePage of the paintable computer to transfer to.</param>
		/// <returns>True if queuing succeded</returns>
		public abstract bool QueueTransfer(int mirrorIndex);

		/// <summary>
		/// Queue a prioritized process fragment transfer ticket.
		/// </summary>
		/// <param name="mirrorIndex">The index of the mirrored Homepage of the Paintable to transfer to.</param>
		/// <param name="priority">Priority</param>
		/// <returns></returns>
		public abstract bool QueueTransfer(int mirrorIndex, byte priority);

		/// <summary>
		/// Used to check if any transfer requests are still pending.
		/// </summary>
		/// <returns>True if requests are still pending</returns>
		public abstract bool IsQueuedForTransfer();

		/// <summary>
		/// Used to check if a specific transfer request to the selected mirror is still pending.
		/// </summary>
		/// <param name="mirrorIndex"></param>
		/// <returns>True if request is still pending</returns>
		public abstract bool IsQueuedForTransfer(int mirrorIndex);

		/// <summary>
		/// Removes the pending transfer ticket to the specified mirror.
		/// </summary>
		/// <param name="mirrorIndex">The index of the mirrored Homepage</param>
		/// <returns>
		/// True if the request ticket has been removed. False if no request exists.
		/// </returns>
		public abstract bool ClearQueueTransfer(int mirrorIndex);

		/// <summary>
		/// Schedules the current executing process fragment for uninstallation.
		/// </summary>
		/// <returns></returns>
		public abstract bool QueueUninstall();

		/// <summary>
		/// Schedules the current executing PFrag for uninstall.
		/// </summary>
		/// <param name="priority">The priority.</param>
		/// <returns>
		/// True if allowed to transfer; false othervise;
		/// </returns>
		public abstract bool QueueUninstall(byte priority);

		/// <summary>
		/// Gets the paintable computer Id.
		/// </summary>
		/// <returns>The Computer Id</returns>
		/// <remarks>Not guaranteed to be unique for all computers.</remarks>
		public abstract uint GetComputerId();

		#endregion
	}
}