/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Text;
using PaintableSimulator.Engine.PaintableComputers.Sensors;
using PaintableSimulator.Engine.PaintableComputers.OperatingSystem;
using System.Drawing;

namespace PaintableSimulator.Engine.PaintableComputers.Sensors
{

	#region Colorizer

	/// <summary>
	/// The homepage colorizer of the pressure sensor
	/// </summary>
	[FriendlyName("Pressure Sensor colorizer")]
	[FriendlyDescription("This colorizer sets the color according to the preassure the sensor reads")]
	public class PressureSensorColorizer : HomepageColorizer
	{

		/// <summary>
		/// Colorizes the specified homepage.
		/// </summary>
		/// <param name="homepage">The homepage.</param>
		/// <returns></returns>
		public override Color Colorize(Homepage homepage)
		{
			Color retval = Color.WhiteSmoke;

			for (int i = 0; i < homepage.Count; i++)
				if (homepage.KeyList[i] == PressureSensorComputer.ProcessFragmentId)
				{
					double pressure = 0.0; 

					if (homepage.DataList[i].Count == 0)
						break;


					// read the pressure from the homepage
					pressure = (double)homepage.DataList[i][0];

					int color = (int)(255 * pressure / 10.0);
					if (color > 255)
						color = 255;

					if (color < 0)
						color = 0;


					retval = Color.FromArgb(color, color, 0);

					break;
				}

			return retval;
		}
	}

	#endregion

	/// <summary>
	/// 
	/// </summary>
	[DefaultHomepageColorizer(typeof(PressureSensorColorizer))]
	public class PressureSensorComputer : GenericPaintableComputer, ISensor
	{
		public const uint ProcessFragmentId = 0x12121212;
		
		/// <summary>
		/// This sensor only works with the simpleTemperatureDataFeeder
		/// </summary>
		private readonly Type[] usesDataFeeders = new Type[] { typeof(AntWalkerDataFeeder) };

		private double pressure;

		#region Constructors

		/// <summary>
		/// Initializes a new instance of the <see cref="T:PressureSensorComputer"/> class.
		/// </summary>
		public PressureSensorComputer()
			: this(DEFAULT_MAXIMUMNEIGHBOURCOUNT)
		{}
		/// <summary>
		/// Initializes a new instance of the <see cref="T:PressureSensorComputer"/> class.
		/// </summary>
		/// <param name="maximumNeighbourCount">The maximum neighbour count.</param>
		public PressureSensorComputer(uint maximumNeighbourCount)
			: base(maximumNeighbourCount)
		{
			// install the pressure sensor private homepage on this PaC
			localHomepage.Add(ProcessFragmentId, new HomepagePostCollection());
		}

		#endregion
		
		#region ISensor Members

		/// <summary>
		/// Sets the environment value.
		/// </summary>
		/// <param name="dataFeederType">Type of the data feeder.</param>
		/// <param name="sensorValue">The sensor value.</param>
		public void SetEnvironmentValue(Type dataFeederType, double sensorValue)
		{
			if (dataFeederType == typeof(AntWalkerDataFeeder))
			{
				pressure = sensorValue;
			}
		}

		/// <summary>
		/// Gets the data feeders this sensor works with.
		/// </summary>
		/// <value>The data feeders.</value>
		public Type[] DataFeeders
		{
			get
			{
                return new Type[] { typeof(AntWalkerDataFeeder) };
			}
		}

		#endregion


		/// <summary>
		/// Cycles this instance.
		/// </summary>
		public override void Cycle()
		{

			// update the homepage with fresh sensor values
			UpdateSensorHomepage();

			// cycle the GenericPaintableComputer
			base.Cycle();

		}



		/// <summary>
		/// Updates the sensor homepage.
		/// </summary>
		private void UpdateSensorHomepage()
		{
			base.PostEntry(ProcessFragmentId, 0, pressure);
		}


	}
}
