/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
#region Using Directives

using System;
using System.ComponentModel;
using System.Drawing;
using PaintableSimulator.Engine;
using PaintableSimulator.Engine.PaintableComputers.OperatingSystem;
using PaintableSimulator.Engine.PaintableComputers;
using System.Globalization;

#endregion

namespace PaintableSimulator.Programs
{

	#region Colorizer

	/// <summary>
	/// 
	/// </summary>
	public class BreadcrumbColorizer : HomepageColorizer
	{
		private static int maximumReachedHops = 1;

		/// <summary>
		/// Colorizes the specified homepage.
		/// </summary>
		/// <param name="homepage">The homepage.</param>
		/// <returns></returns>
		public override Color Colorize(Homepage homepage)
		{
			Color retval = Color.WhiteSmoke;

			for (int i = 0; i < homepage.Count; i++)
				if (homepage.KeyList[i] == 0xFFA4F148)
					if (homepage.DataList[i].Count > 0)
					{
						int hop = (int)homepage.DataList[i][0];

						// store the maximum distance
						if (maximumReachedHops < hop)
						{
							maximumReachedHops = hop;
						}
						else
						{
							int green = (int)((1 - (double)hop / (double)maximumReachedHops) * 200 + 50);
							if (green < 0)
							{
								green = 0;
							}
							else if (green > 255)
								green = 255;

							retval = Color.FromArgb(0, green, 0);
						}
					}

			return retval;
		}
	}

	#endregion

	/// <summary>
	/// 
	/// </summary>
	[Serializable()]
	[DefaultHomepageColorizerAttribute(typeof(BreadcrumbColorizer))]
	public sealed class Breadcrumb : ProcessFragmentBase
	{
		#region Process Fragment Identity

		public override uint ProcessFragmentId
		{
			get { return 0xFFA4F148; }
		}

		#endregion

		#region Breadcrumb Local state

		private int crumbNumber;

		/// <summary>
		/// Gets or sets the crumb number.
		/// </summary>
		/// <value>The crumb number.</value>
		[Browsable(true)]
		public int CrumbNumber
		{
			get { return crumbNumber; }
			set { crumbNumber = value; }
		}

		#endregion

		/// <summary>
		/// This method is called when the PFrag first enters a Paintable.
		/// </summary>
		/// <param name="initialState">The initial state vector passed on by his ancestor.</param>
		/// <remarks>
		/// </remarks>
		public override void Install()
		{
			if (Api.GetPostCount(this.ProcessFragmentId) == 0)
			{
				// everything is ok, we are allowed to install
				// post the initial Crumb number to the HP
				Api.PostEntry(ProcessFragmentId, this.CrumbNumber); // at index 0
			}
		}

		/// <summary>
		/// This method is called after the PFrag queued himself for uninstall and the OS approves an uninstall.
		/// </summary>
		/// <param name="remainingTimeslots"></param>
		/// <remarks>
		/// Here you sould do some pre uninstall processing.
		/// Removing posts from the homepage is unneccesary because the os does that for you automatically.
		/// </remarks>
		public override void Uninstall(byte remainingTimeslots)
		{
		}


		/// <summary>
		/// This method is called if the PFrag requested a transfer to the neighboring particles and
		/// his request was approved.
		/// </summary>
		/// <param name="copy">The copy of the ProcessFragmentBase.</param>
		public override void TransferGranted(ref ProcessFragmentBase copy)
		{
			Breadcrumb breadCrumbCopy = (Breadcrumb)copy;

			breadCrumbCopy.CrumbNumber = this.CrumbNumber + 1;
		}

		/// <summary>
		/// This method is called if the PFragment requested a transfer to the neighboring particles and
		/// his request was denied.
		/// </summary>
		public override void TransferRefused()
		{
		}

		/// <summary>
		/// The main processing method for Process Fragments
		/// </summary>
		/// <param name="remainingTimeslots">A value indicating the available processing time for the Update method.</param>
		/// <remarks>
		/// The time slot of 0 indicates less time available for processing and 255 more time.
		/// This is jst a suggested value, it has no bearing on the execution of the Update method (yet).
		/// </remarks>
		public override void Update(byte remainingTimeslots)
		{
			bool emptyNeighbours = false;

			// check for PaC's without BreadCrumbs
			int mirrorCount = Api.GetIOSpaceCount();

			if (mirrorCount <= 0)
				return;

			for (int i = 0; i < mirrorCount; i++)
				if (Api.GetIOPostCount(i, ProcessFragmentId) == 0)
				{
					emptyNeighbours = true;
					if (!Api.IsQueuedForTransfer(i))
						Api.QueueTransfer(i);
				}

			if (emptyNeighbours)
				return;

			// if all PaC's contain bread crumbs select the smallest
			int neighbourhoodMin = int.MaxValue;
			for (int i = 1; i < mirrorCount; i++)
			{
				object post = Api.GetIOPost(i, ProcessFragmentId, 0);
				if (post != null)
				{
					int mirrorPost = (int)post;
					if (neighbourhoodMin > mirrorPost)
						neighbourhoodMin = mirrorPost;
				}
			}

			//termination condition
			if (neighbourhoodMin > this.CrumbNumber)
			{
				// Ovo ne pise u knjizi ali mislim da bi trebalo... barem zanimljivije izgleda
				// ako se ova naredba makne, sve radi ali na trenutni pfrag se vise ne useljava novi
				// breadcrumb jer vec postoji post na ovom PaC-u

				//Api.RemoveAllEntries(this.ProcessFragmentId);

				Api.QueueUninstall();

				return;
			}
			else
			{
				this.CrumbNumber = neighbourhoodMin + 1;

				//repost
				Api.PostEntry(ProcessFragmentId, 0, this.CrumbNumber);
			}
		}

	}
}