/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using PaintableSimulator.Engine;
using System.Collections.ObjectModel;
using System.Drawing.Drawing2D;
using System.Threading;
using System.Runtime.Remoting;
using System.Net.Sockets;
using System.Diagnostics;
using System.Drawing.Imaging;

namespace PaintableSimulator.SmartClient
{
	public partial class SimulatorClient
	{

		#region Display info

		private class DisplayInfo
		{
			public Bitmap Buffer;
			public FriendlyType ColorizerType;
			public float ScaleFactor;
			public Size ClientSize;
			public Size WallSize;
			public Point DisplayOffset;
		}

		#endregion

		/// <summary>
		/// 
		/// </summary>
		private DisplayInfo displayInfo = new DisplayInfo();

		/// <summary>
		/// Creates the computer circle.
		/// </summary>
		/// <param name="center">The center.</param>
		/// <param name="radius">The radius.</param>
		/// <returns></returns>
		private Rectangle CreateCircleAroundPoint(Point center, uint radius)
		{
			int left = center.X - (int)radius;
			int top = center.Y - (int)radius;
			int width = 2 * (int)radius;
			int height = 2 * (int)radius;

			return new Rectangle(left, top, width, height);

		}

		/// <summary>
		/// Draws the visualisation data.
		/// </summary>
		/// <param name="graphics">The graphics.</param>
		private void DrawVisualisationData(Graphics graphics)
		{

			if (IsSimulatorDataAvailable())
			{
				if ((computerColors == null) || (computerColors.Count == 0) ||
					(completeComputerDataList == null) || (completeComputerDataList.Count == 0))
					return;

				int scaledWallWidth = (int)(displayInfo.WallSize.Width * displayInfo.ScaleFactor);
				int scaledWallHeight = (int)(displayInfo.WallSize.Height * displayInfo.ScaleFactor);

				int deltax = scaledWallWidth - displayInfo.ClientSize.Width;
				int deltay = scaledWallHeight - displayInfo.ClientSize.Height;

				int dx = (-displayInfo.DisplayOffset.X) - (deltax / 2);
				int dy = (-displayInfo.DisplayOffset.Y) - (deltay / 2);

				graphics.TranslateTransform(dx, dy);
				graphics.ScaleTransform(displayInfo.ScaleFactor, displayInfo.ScaleFactor);

				graphics.Clear(Color.Black);

				// draw computers
				for (int i = 0; i < completeComputerDataList.Count; i++)
				{
					if (completeComputerDataList[i].IsPortal)
					{
						Rectangle area = CreateCircleAroundPoint(completeComputerDataList[i].Location, 7);
						Brush areaBrush = new SolidBrush(Color.FromArgb(200, Color.Yellow));
						graphics.FillEllipse(areaBrush, area);

					}

					Rectangle circle = CreateCircleAroundPoint(completeComputerDataList[i].Location, 3);

					// draw the computer circle
					Brush fillBrush = new SolidBrush(computerColors[i]);
					graphics.FillEllipse(fillBrush, circle);

				}

			}

		}


		/// <summary>
		/// Returns the scale factor.
		/// </summary>
		/// <returns></returns>
		private float ScaleFactor()
		{
			string zoom = zoomSelection.Text;
			if (zoom.EndsWith("%"))
				zoom = zoom.Substring(0, zoom.Length - 1);

			float scale = 1.0f;
			try
			{
				scale = float.Parse(zoom) / 100;
			}
			catch
			{ }

			return scale;
		}


		/// <summary>
		/// Recalculates the scroll bar limits.
		/// </summary>
		private void RecalculateScrollBarLimits()
		{

			// recalculate scrollbar values
			int scaledWallWidth = (int)(displayInfo.WallSize.Width * displayInfo.ScaleFactor);
			int scaledWallHeight = (int)(displayInfo.WallSize.Height * displayInfo.ScaleFactor);

			int deltax = scaledWallWidth - displayInfo.ClientSize.Width;
			int deltay = scaledWallHeight - displayInfo.ClientSize.Height;

			if (deltax >= 0)
			{
				displayHorizontalScroll.Minimum = -deltax / 2;
				displayHorizontalScroll.Maximum = deltax / 2;
				displayHorizontalScroll.Enabled = true;
			}
			else
			{
				displayHorizontalScroll.Enabled = false;
			}

			if (deltay >= 0)
			{

				displayVerticalScroll.Minimum = -deltay / 2;
				displayVerticalScroll.Maximum = deltay / 2;
				displayVerticalScroll.Enabled = true;

			}
			else
			{
				displayVerticalScroll.Enabled = false;
			}


			// reset scrollbar values by keeping the last viewport visible
			if (deltax >= 0)
				displayHorizontalScroll.Value = (int)(displayHorizontalScroll.Value * displayInfo.ScaleFactor);
			else
				displayHorizontalScroll.Value = 0;

			if (deltay >= 0)
				displayVerticalScroll.Value = (int)(displayVerticalScroll.Value * displayInfo.ScaleFactor);
			else
				displayVerticalScroll.Value = 0;

		}


		/// <summary>
		/// Handles the Resize event of the simulatorDisplay control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void simulatorDisplay_Resize(object sender, EventArgs e)
		{
			displayInfo.ClientSize = simulatorDisplay.ClientSize;

			if (displayInfo.ClientSize.IsEmpty)
				return;


			displayInfo.Buffer = new Bitmap(displayInfo.ClientSize.Width, displayInfo.ClientSize.Height, PixelFormat.Format32bppArgb);

			if (!IsSimulatorDataAvailable())
				return;

			RecalculateScrollBarLimits();
		}


		/// <summary>
		/// Transforms the coordinate from page to world.
		/// </summary>
		/// <param name="input">The input.</param>
		/// <returns></returns>
		public Point TransformCoordinatePageToWorld(Point input)
		{
			Point[] pts = new Point[] { input };

			Graphics g = null;

			lock (displaySyncRoot)
			{
				g = Graphics.FromImage(displayInfo.Buffer);
			}

			int scaledWallWidth = (int)(displayInfo.WallSize.Width * displayInfo.ScaleFactor);
			int scaledWallHeight = (int)(displayInfo.WallSize.Height * displayInfo.ScaleFactor);

			int deltax = scaledWallWidth - displayInfo.ClientSize.Width;
			int deltay = scaledWallHeight - displayInfo.ClientSize.Height;

			int ddx = (-displayInfo.DisplayOffset.X) - (deltax / 2);
			int ddy = (-displayInfo.DisplayOffset.Y) - (deltay / 2);

			g.TranslateTransform(ddx, ddy);
			g.ScaleTransform(displayInfo.ScaleFactor, displayInfo.ScaleFactor);

			g.TransformPoints(CoordinateSpace.World, CoordinateSpace.Page, pts);

			g.Dispose();

			return pts[0];
		}

		/// <summary>
		/// Handles the ValueChanged event of the displayScroll control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void displayScroll_Changed(object sender, EventArgs e)
		{
			if (sender == displayVerticalScroll)
				displayInfo.DisplayOffset.Y = displayVerticalScroll.Value;

			if (sender == displayHorizontalScroll)
				displayInfo.DisplayOffset.X = displayHorizontalScroll.Value;
		}

		/// <summary>
		/// Handles the Scroll event of the displayScroll control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.ScrollEventArgs"/> instance containing the event data.</param>
		private void displayScroll_Scroll(object sender, ScrollEventArgs e)
		{
			if (sender == displayVerticalScroll)
				displayInfo.DisplayOffset.Y = displayVerticalScroll.Value;

			if (sender == displayHorizontalScroll)
				displayInfo.DisplayOffset.X = displayHorizontalScroll.Value;
		}

	}
}
